import aiohttp
import logging
import datetime

from typing import List
from dataclasses import dataclass

logging.basicConfig(level=logging.DEBUG,
                    format="%(levelname)s:%(asctime)s:%(message)s")

COLOR_PALLETE = {
    "red": 0xFD0061,
    "green": 0x00D166,
    "blue": 0x0099E1,
    "orange": 0xF93A2F,
    "gray": 0x979C9F,
    "purple1": 0xA652BB,
    "purple2": 0x7A2F8F,
}


def get_embed(title: str, color: str, fields: List):
    embed = {
        "title": title,
        "timestamp": str(datetime.datetime.utcnow()),
        "color": COLOR_PALLETE[color],
        "fields": fields,
    }
    return embed


@dataclass
class DiscordBot:
    botname: str
    url: str

    async def send_discord_message(self, embeds: List):
        try:

            payload = {
                "username": self.botname,
                "embeds": embeds,
            }
            async with aiohttp.ClientSession() as session:
                async with session.post(self.url, json=payload) as resp:
                    logging.info(f"Sending Payload To Discord - {resp.status}")
                    text = await resp.text()
                    logging.debug(text)
        except Exception as err:
            logging.debug(err)
            raise err

    async def send_embeded_field(self, field_name: str, field_value: str,  description: str):
        try:
            field = [
                {
                    "name": field_name,
                    "value": field_value,
                }
            ]
            embed = get_embed(title=description, color="orange", fields=field)
            await self.send_discord_message(embeds=[embed])
            logging.info("Sent New Transaction to discord")
        except Exception as err:
            logging.debug(err)
            raise err

    # async def send_balance(self, wallets: List[Wallet], balance: Balance):
    #     try:
    #         fields = []
    #         balance_field = {"name": "General Balance", "value": str(balance)}
    #         fields.append(balance_field)

    #         for wallet in wallets:
    #             wallet_field = {
    #                 "name": wallet.exchange.upper(), "value": str(wallet)}
    #             fields.append(wallet_field)

    #         embed = get_embed(title="New Balance",
    #                           color="purple2", fields=fields)

    #         await self.send_discord_message(embeds=[embed])
    #         logging.info("Sent Balance to Discord")
    #     except Exception as err:
    #         logging.debug(err)
    #         raise err

    # async def send_error_message(self, microservice: str, error_message: str):
    #     try:
    #         fields = [
    #             {
    #                 "name": f"Error in {microservice}",
    #                 "value": f" Message: {error_message}",
    #             }
    #         ]

    #         embed = get_embed(title="Error Message",
    #                           color="red", fields=fields)
    #         await self.send_discord_message(embeds=[embed])
    #         logging.info("Sent Error Message")

    #     except Exception as err:
    #         logging.debug(err)
    #         raise err

    # async def send_unbalance(self, unbalanced_field: str, target_exchange: str):
    #     try:
    #         fields = [
    #             {
    #                 "name": f"Unbalanced in { target_exchange.upper() }",
    #                 "value": unbalanced_field,
    #             }
    #         ]
    #         embed = get_embed(title="Unbalanced Wallet",
    #                           color="red", fields=fields)
    #         await self.send_discord_message(embeds=[embed])
    #         logging.info(f"Sent unbalance to discord")
    #     except Exception as err:
    #         logging.debug(err)
    #         raise err



import random
import ujson
import asyncio
import nats
import logging

from nats.aio.client import Client as NATS
from nats.js.client import JetStream

from typing import Callable, Dict, Optional, List
from pydantic import BaseModel


class JetStreamMessagingBot(BaseModel):
    botname: str
    nats_url: str
    nc: Optional[NATS]
    js: Optional[JetStream]

    class Config:
        arbitrary_types_allowed = True

    async def connect(self, timeout: int = 10):
        try:
            self.nc = await nats.connect(servers=[self.nats_url])
            logging.info(f"NATS: Connected")
            self.js = self.nc.jetstream(timeout=timeout)
            logging.info(f"NATS: JetStream {self.botname} Created")
        except TimeoutError as err:
            logging.debug(err)
            raise err



import json
import random
import asyncio
import nats
import logging

from dataclasses import dataclass, field

from nats.aio.client import Client as NATS
from stan.aio.client import Client as STAN

logging.basicConfig(level=logging.DEBUG,
                    format="%(levelname)s:%(asctime)s:%(message)s")


@dataclass
class MessagingBot:
    botname: str
    nats_url: str
    stan_cluster_id: str
    stan_client_id: str
    pod_name: str
    nc: NATS = field(default_factory=NATS)
    sc: STAN = field(default_factory=STAN)

    async def connect_nats_streaming(self):
        logging.info("Connecting Bot to NATS Streaming")
        try:
            # random_number = random.randint(0, 1000)
            await self.nc.connect(servers=[self.nats_url])
            await self.sc.connect(
                cluster_id=self.stan_cluster_id,
                client_id=f"{self.stan_client_id}_{self.pod_name}",
                nats=self.nc,
            )
            logging.info("NATS Streaming Connected")
        except Exception as err:
            logging.debug(f"Could not conenct to NATS")
            logging.error(err)

    async def cb_ack(self, message):
        logging.info(f"Recived ack: {message.guid}")

    async def subscribe(self, subject, durable_name=None, cb=None, start_at=None):
        sc = self.sc
        await sc.subscribe(
            subject=subject,
            durable_name=durable_name,
            cb=cb,
            manual_acks=True,
            start_at=start_at,
        )

    async def publish(self, subject, payload):
        sc = self.sc
        await sc.publish(subject=subject, payload=payload, ack_handler=self.cb_ack)

    async def close(self):
        await self.sc.close()
        await self.nc.close()

    async def unsuscribe(self):
        await self.sc.unsubscribe()

    async def retry_message(self, subject, failed_message):
        new_message = json.dumps(failed_message).encode()
        await asyncio.sleep(self.WAIT_RETRY_MESSAGE)
        await self.publish(subject, new_message)

    async def pong_callback(self, message):
        nc = self.nc
        response = f"{self.botname} - PONG"
        await nc.publish(message.reply, response.encode(encoding="UTF-8"))

    async def await_for_ping(self, microservice_channel: str):
        nc = self.nc
        await nc.subscribe(subject=microservice_channel, cb=self.pong_callback)

    async def request_pong(self, microservice_channel: str):
        try:
            data = f"{self.botname} - PING"
            future = self.nc.request(
                microservice_channel, data.encode(), timeout=10)
            logging.info(
                f"Liveliness: Asking {microservice_channel} for Response")
            msg = await future
            subject = msg.subject
            data = msg.data.decode()
            logging.info(
                f"Liveliness: Obtained Response from {microservice_channel}")
            logging.info(subject)
            return True
        except nats.aio.errors.ErrTimeout:
            logging.error(
                f"Liveliness: Did not receive response from {microservice_channel}")
            return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['odin_messaging_bot']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.3.1,<4.0.0',
 'asyncio-nats-client==0.11.4',
 'asyncio-nats-streaming>=0.4.0,<0.5.0',
 'nest-asyncio>=1.5.1,<2.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'python-dotenv>=0.19.1,<0.20.0',
 'ujson>=4.3.0,<5.0.0']

setup_kwargs = {
    'name': 'odin-messaging-bot',
    'version': '0.1.10',
    'description': 'A NATS/STAN Bot to publish/subscribe messages.',
    'long_description': None,
    'author': 'adolfrodeno',
    'author_email': 'amvillalobos@uc.cl',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
